@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2021 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -63,9 +63,8 @@ function preciseTransform(model, num)
     return
   end
 
-  if num==3 and not model.snap.with_axes then
-    model:warning("Cannot mirror at x-axis",
-		  "The coordinate system has not been set")
+  if (num==3 or num==4) and not model.snap.with_axes then
+    model:warning("Cannot mirror at axis", "The coordinate system has not been set")
     return
   end
 
@@ -88,13 +87,17 @@ function preciseTransform(model, num)
     matrix = (ipe.Rotation(model.snap.orientation)
 	    * ipe.Matrix(1, 0, 0, -1, 0, 0)
 	  * ipe.Rotation(-model.snap.orientation))
-  elseif num == 4 then   -- turn 90 degrees
+  elseif num == 4 then -- Mirror at y-axis
+    matrix = (ipe.Rotation(model.snap.orientation)
+	    * ipe.Matrix(-1, 0, 0, 1, 0, 0)
+	  * ipe.Rotation(-model.snap.orientation))
+  elseif num == 5 then   -- turn 90 degrees
     matrix = ipe.Matrix(0, 1, -1, 0, 0, 0)
-  elseif num == 5 then   -- turn 180 degrees
+  elseif num == 6 then   -- turn 180 degrees
     matrix = ipe.Matrix(-1, 0, 0, -1, 0, 0)
-  elseif num == 6 then   -- turn 270 degrees
+  elseif num == 7 then   -- turn 270 degrees
     matrix = ipe.Matrix(0, -1, 1, 0, 0, 0)
-  elseif num == 7 then   -- rotate by angle
+  elseif num == 8 then   -- rotate by angle
     local str = model:getString("Enter angle in degrees")
     if not str or str:match("^%s*$") then return end
     local degrees = tonumber(str)
@@ -104,7 +107,7 @@ function preciseTransform(model, num)
     end
     matrix = ipe.Rotation(math.pi * degrees / 180.0)
     label = "rotation by " .. degrees .. " degrees"
-  elseif num == 8 then   -- stretch or scale
+  elseif num == 9 then   -- stretch or scale
     local str = model:getString("Enter stretch factors")
     if not str or str:match("^%s*$") then return end
     if str:match("^[%+%-%d%.]+$") then
@@ -133,6 +136,11 @@ function preciseTransform(model, num)
       label = "stretch by " .. sx .. ", " .. sy
       matrix = ipe.Matrix(sx, 0, 0, sy, 0, 0)
     end
+    if model.snap.with_axes then
+      matrix = (ipe.Rotation(model.snap.orientation)
+	      * matrix
+	      * ipe.Rotation(-model.snap.orientation))
+    end
   end
 
   local origin
@@ -193,12 +201,21 @@ function preciseBox(model)
     return
   end
   local sx, sy = tonumber(ssx), tonumber(ssy)
-  local shape = { boxshape(V(0,0), V(sx * dpmm, sy * dpmm)) }
+  local corner = V(sx * dpmm, sy * dpmm)
+  local origin = V(0,0)
+  if model.snap.with_axes then
+    origin = model.snap.origin
+  end
+  local shape = { boxshape(origin, origin + corner) }
   local obj = ipe.Path(model.attributes, shape)
   model:creation("create precise box", obj)
 end
 
 function boundingBox(model)
+  if not model:page():hasSelection() then
+    model.ui:explain("no selection")
+    return
+  end
   local box = bounding_box(model:page())
   local shape = { boxshape(box:bottomLeft(), box:topRight()) }
   local obj = ipe.Path(model.attributes, shape)
@@ -350,6 +367,7 @@ methods = {
   { label = "Mirror horizontal", run=preciseTransform },
   { label = "Mirror vertical", run=preciseTransform },
   { label = "Mirror at x-axis", run=preciseTransform },
+  { label = "Mirror at y-axis", run=preciseTransform },
   { label = "Turn 90 degrees", run=preciseTransform },
   { label = "Turn 180 degrees", run=preciseTransform },
   { label = "Turn 270 degrees", run=preciseTransform },
