@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2021 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -40,6 +40,8 @@ This ipelet is part of Ipe.
 
 V = ipe.Vector
 current_symbol = nil
+current_group = nil
+current_element = nil
 
 ----------------------------------------------------------------------
 
@@ -88,14 +90,54 @@ end
 
 function select_symbol(model)
   local s = model.doc:sheets():allNames("symbol")
+  local symbolsByGroup = {}
+  for _, sym in ipairs(s) do
+    local i = sym:find("/")
+    local g, n
+    if i then
+      g = sym:sub(1, i-1)
+      n = sym:sub(i+1)
+    else
+      g = ""
+      n = sym
+    end
+    if not symbolsByGroup[g] then
+      symbolsByGroup[g] = {}
+    end
+    local sbg = symbolsByGroup[g]
+    sbg[#sbg+1] = n
+  end
+  local groups = {}
+  for g,sbg in pairs(symbolsByGroup) do
+    groups[#groups+1] = g
+  end
+  table.sort(groups)
   local d = ipeui.Dialog(model.ui:win(), "Use symbol")
+  groups.action = function (d)
+    		    local g = d:get("group")
+		    if g ~= current_group then
+		       current_group = g
+		       local syms = symbolsByGroup[groups[g]]
+		       d:set("select", syms)
+		    end
+		  end
   d:add("label", "label", { label = "Select symbol" }, 1, 1, 1, 3)
-  d:add("select", "combo", s, 2, 1, 1, 3)
+  d:add("group", "combo", groups, 2, 1, 1, 3)
+  if current_group == nil then current_group = 1 end
+  d:set("group", current_group)
+  d:add("select", "combo", symbolsByGroup[groups[current_group]], 3, 1, 1, 3)
+  if current_element then
+    d:set("select", current_element)
+  end
   d:addButton("ok", "&Ok", "accept")
   d:addButton("cancel", "&Cancel", "reject")
   d:setStretch("column", 1, 1)
   if not d:execute() then return end
-  return s[d:get("select")]
+  current_element = d:get("select")
+  local g = groups[current_group]
+  local sym = symbolsByGroup[g][current_element]
+  if g ~= "" then g = g .. "/" end
+  return g .. sym
 end
 
 function use_symbol(model, num)
