@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2020 Otfried Cheong
+    Copyright (c) 1993-2021 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -148,6 +148,10 @@ prefs.win_mini_edit = false
 -- Should LaTeX be run automatically every time text has changed?
 prefs.auto_run_latex = true
 
+-- Allow Ipe to freeze while LaTeX is running
+-- Useful when your Latex is fast and you want to keep typing while Latex is running
+prefs.freeze_in_latex = false
+
 -- Should the external editor be called automatically?
 prefs.auto_external_editor = nil
 
@@ -167,8 +171,15 @@ prefs.editor_size = { 0, 0 }
 prefs.width_notes_bookmarks = 200
 
 -- Size of main window at startup
+-- You can also determine the position of the main window by adding x and y coordinates,
+-- like so: { 1160, 720, 500, 300 }
 prefs.window_size = { 1160, 720 }
 
+-- Should Ipe start maximized or even in full screen mode? (Windows and Qt only)
+prefs.start_screen = "normal"
+--prefs.start_screen = "maximized"
+--prefs.start_screen = "full"
+
 -- Size of Latex error message window
 prefs.latexlog_size = { 800, 600 }
 
@@ -178,6 +189,9 @@ prefs.page_sorter_size = { 960, 600 }
 -- Width of page thumbnails (height is computed automatically)
 prefs.thumbnail_width = 200
 
+-- Number of recent files remembered
+prefs.num_recent_files = 12
+
 -- The laser pointer
 prefs.laser_pointer = {
   color = { r = 1.0, g = 0.0, b = 0.0 }, -- red
