@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (c) 1993-2019 Otfried Cheong
+    Copyright (c) 1993-2021 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -168,6 +168,10 @@ function MODEL:insertOpacity(m, obj)
   local opacityL = self.doc:sheets():allNames("opacity")
   local opacity = obj:get("opacity")
   m:add("opacity", "Opacity: " .. opacity, opacityL, nil, opacity)
+  if obj:type() == "path" then
+    local strokeOpacity = obj:get("strokeopacity")
+    m:add("strokeopacity", "Stroke opacity: " .. strokeOpacity, opacityL, nil, strokeOpacity)
+  end
 end
 
 ----------------------------------------------------------------------
@@ -338,10 +342,10 @@ function MODEL:properties_text(obj, m)
   else
     local ts = obj:get("labelstyle")
     m:add("labelstyle", "Style: " .. ts, labelstyleL, nil, ts)
-    local ha = obj:get("horizontalalignment")
-    m:add("horizontalalignment", "Horizontal alignment: " .. ha,
-	  {"left", "right", "hcenter"}, nil, ha)
   end
+  local ha = obj:get("horizontalalignment")
+  m:add("horizontalalignment", "Horizontal alignment: " .. ha,
+	{"left", "right", "hcenter"}, nil, ha)
   local va = obj:get("verticalalignment")
   m:add("verticalalignment", "Vertical alignment: " .. va,
 	{"bottom", "baseline", "top", "vcenter"}, nil, va)
@@ -364,6 +368,7 @@ function MODEL:properties_image(obj, m)
 				  info.width, info.height))
   m:add("comment", "Format " .. info.format)
   self:insertBasic(m, obj)
+  self:insertOpacity(m, obj)
 end
 
 function MODEL:properties_reference(obj, m)
@@ -460,6 +465,7 @@ function MODEL:multiAttributes(m, tm)
 
   if tm.reference then m:add("symbolsize", "Size", sizes) end
   if tm.text or tm.path then m:add("opacity", "Opacity", opacity) end
+  if tm.path then m:add("strokeopacity", "Stroke opacity", opacity) end
   m:add("action_apply_properties", "Apply properties")
 end
 
@@ -571,6 +577,7 @@ function MODEL:pick_properties_path(obj)
   a.tiling = obj:get("tiling")
   a.gradient = obj:get("gradient")
   a.opacity = obj:get("opacity")
+  a.strokeopacity = obj:get("strokeopacity")
 end
 
 function MODEL:pick_properties_image(obj)
@@ -656,6 +663,7 @@ function apply_properties_path(obj, a)
   obj:set("tiling", a.tiling)
   obj:set("gradient", a.gradient)
   obj:set("opacity", a.opacity)
+  obj:set("strokeopacity", a.strokeopacity)
 end
 
 function apply_properties_reference(obj, a)
